@@ -288,6 +288,9 @@ class AFMLayer(nn.Module):
         for tensor in [self.attention_W, self.projection_h, self.projection_p]:
             nn.init.xavier_normal_(tensor, )
 
+        for tensor in [self.attention_b]:
+            nn.init.zeros_(tensor, )
+
         self.dropout = nn.Dropout(dropout_rate)
 
         self.to(device)
@@ -411,17 +414,29 @@ class CrossNet(nn.Module):
         - **in_features** : Positive integer, dimensionality of input features.
         - **input_feature_num**: Positive integer, shape(Input tensor)[-1]
         - **layer_num**: Positive integer, the cross layer number
+        - **parameterization**: string, ``"vector"``  or ``"matrix"`` ,  way to parameterize the cross network.
         - **l2_reg**: float between 0 and 1. L2 regularizer strength applied to the kernel weights matrix
         - **seed**: A Python integer to use as random seed.
       References
         - [Wang R, Fu B, Fu G, et al. Deep & cross network for ad click predictions[C]//Proceedings of the ADKDD'17. ACM, 2017: 12.](https://arxiv.org/abs/1708.05123)
+        - [Wang R, Shivanna R, Cheng D Z, et al. DCN-M: Improved Deep & Cross Network for Feature Cross Learning in Web-scale Learning to Rank Systems[J]. 2020.](https://arxiv.org/abs/2008.13535)
     """
 
-    def __init__(self, in_features, layer_num=2, seed=1024, device='cpu'):
+    def __init__(self, in_features, layer_num=2, parameterization='vector', seed=1024, device='cpu'):
         super(CrossNet, self).__init__()
         self.layer_num = layer_num
-        self.kernels = torch.nn.ParameterList(
-            [nn.Parameter(nn.init.xavier_normal_(torch.empty(in_features, 1))) for i in range(self.layer_num)])
+        self.parameterization = parameterization
+        if self.parameterization == 'vector':
+            # weight in DCN.  (in_features, 1)
+            self.kernels = torch.nn.ParameterList(
+                [nn.Parameter(nn.init.xavier_normal_(torch.empty(in_features, 1))) for i in range(self.layer_num)])
+        elif self.parameterization == 'matrix':
+            # weight matrix in DCN-M.  (in_features, in_features)
+            self.kernels = torch.nn.ParameterList([nn.Parameter(nn.init.xavier_normal_(
+                torch.empty(in_features, in_features))) for i in range(self.layer_num)])
+        else:  # error
+            raise ValueError("parameterization should be 'vector' or 'matrix'")
+
         self.bias = torch.nn.ParameterList(
             [nn.Parameter(nn.init.zeros_(torch.empty(in_features, 1))) for i in range(self.layer_num)])
         self.to(device)
@@ -430,13 +445,98 @@ class CrossNet(nn.Module):
         x_0 = inputs.unsqueeze(2)
         x_l = x_0
         for i in range(self.layer_num):
-            xl_w = torch.tensordot(x_l, self.kernels[i], dims=([1], [0]))
-            dot_ = torch.matmul(x_0, xl_w)
-            x_l = dot_ + self.bias[i] + x_l
+            if self.parameterization == 'vector':
+                xl_w = torch.tensordot(x_l, self.kernels[i], dims=([1], [0]))
+                dot_ = torch.matmul(x_0, xl_w)
+                x_l = dot_ + self.bias[i]
+            elif self.parameterization == 'matrix':
+                dot_ = torch.matmul(self.kernels[i], x_l)  # W * xi  (bs, in_features, 1)
+                dot_ = dot_ + self.bias[i]  # W * xi + b
+                dot_ = x_0 * dot_  # x0 · (W * xi + b)  Hadamard-product
+            else:  # error
+                print("parameterization should be 'vector' or 'matrix'")
+                pass
+            x_l = dot_ + x_l
         x_l = torch.squeeze(x_l, dim=2)
         return x_l
 
 
+class CrossNetMix(nn.Module):
+    """The Cross Network part of DCN-Mix model, which improves DCN-M by:
+      1 add MOE to learn feature interactions in different subspaces
+      2 add nonlinear transformations in low-dimensional space
+      Input shape
+        - 2D tensor with shape: ``(batch_size, units)``.
+      Output shape
+        - 2D tensor with shape: ``(batch_size, units)``.
+      Arguments
+        - **in_features** : Positive integer, dimensionality of input features.
+        - **low_rank** : Positive integer, dimensionality of low-rank sapce.
+        - **num_experts** : Positive integer, number of experts.
+        - **layer_num**: Positive integer, the cross layer number
+        - **device**: str, e.g. ``"cpu"`` or ``"cuda:0"``
+      References
+        - [Wang R, Shivanna R, Cheng D Z, et al. DCN-M: Improved Deep & Cross Network for Feature Cross Learning in Web-scale Learning to Rank Systems[J]. 2020.](https://arxiv.org/abs/2008.13535)
+    """
+
+    def __init__(self, in_features, low_rank=32, num_experts=4, layer_num=2, device='cpu'):
+        super(CrossNetMix, self).__init__()
+        self.layer_num = layer_num
+        self.num_experts = num_experts
+
+        # U: (in_features, low_rank)
+        self.U_list = torch.nn.ParameterList([nn.Parameter(nn.init.xavier_normal_(
+            torch.empty(num_experts, in_features, low_rank))) for i in range(self.layer_num)])
+        # V: (in_features, low_rank)
+        self.V_list = torch.nn.ParameterList([nn.Parameter(nn.init.xavier_normal_(
+            torch.empty(num_experts, in_features, low_rank))) for i in range(self.layer_num)])
+        # C: (low_rank, low_rank)
+        self.C_list = torch.nn.ParameterList([nn.Parameter(nn.init.xavier_normal_(
+            torch.empty(num_experts, low_rank, low_rank))) for i in range(self.layer_num)])
+        self.gating = nn.ModuleList([nn.Linear(in_features, 1, bias=False) for i in range(self.num_experts)])
+
+        self.bias = torch.nn.ParameterList([nn.Parameter(nn.init.zeros_(
+            torch.empty(in_features, 1))) for i in range(self.layer_num)])
+        self.to(device)
+
+    def forward(self, inputs):
+        x_0 = inputs.unsqueeze(2)  # (bs, in_features, 1)
+        x_l = x_0
+        for i in range(self.layer_num):
+            output_of_experts = []
+            gating_score_of_experts = []
+            for expert_id in range(self.num_experts):
+                # (1) G(x_l)
+                # compute the gating score by x_l
+                gating_score_of_experts.append(self.gating[expert_id](x_l.squeeze(2)))
+
+                # (2) E(x_l)
+                # project the input x_l to $\mathbb{R}^{r}$
+                v_x = torch.matmul(self.V_list[i][expert_id].T, x_l)  # (bs, low_rank, 1)
+
+                # nonlinear activation in low rank space
+                v_x = torch.tanh(v_x)
+                v_x = torch.matmul(self.C_list[i][expert_id], v_x)
+                v_x = torch.tanh(v_x)
+
+                # project back to $\mathbb{R}^{d}$
+                uv_x = torch.matmul(self.U_list[i][expert_id], v_x)  # (bs, in_features, 1)
+
+                dot_ = uv_x + self.bias[i]
+                dot_ = x_0 * dot_  # Hadamard-product
+
+                output_of_experts.append(dot_.squeeze(2))
+
+            # (3) mixture of low-rank experts
+            output_of_experts = torch.stack(output_of_experts, 2)  # (bs, in_features, num_experts)
+            gating_score_of_experts = torch.stack(gating_score_of_experts, 1)  # (bs, num_experts, 1)
+            moe_out = torch.matmul(output_of_experts, gating_score_of_experts.softmax(1))
+            x_l = moe_out + x_l  # (bs, in_features, 1)
+
+        x_l = x_l.squeeze()  # (bs, in_features)
+        return x_l
+
+
 class InnerProductLayer(nn.Module):
     """InnerProduct Layer used in PNN that compute the element-wise
     product or inner product between feature vectors.
