@@ -4,6 +4,7 @@ from .xdeepfm import xDeepFM
 from .afm import AFM
 from .autoint import AutoInt
 from .dcn import DCN
+from .dcnmix import DCNMix
 from .fibinet import FiBiNET
 from .nfm import NFM
 from .mlr import MLR
@@ -11,6 +12,4 @@ from .onn import ONN
 from .pnn import PNN
 from .ccpm import CCPM
 from .dien import DIEN
-from .din import DIN
-
-NFFM = ONN
\ No newline at end of file
+from .din import DIN
\ No newline at end of file
